﻿/*
 * Copyright (C) 2012  Manatee Works, Inc.
 *
 */

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace MWBDemo
{
    class PointF
    {
        public float x;
        public float y;
    }

    class MWLocation
    {
        public PointF p1;
        public PointF p2;
        public PointF p3;
        public PointF p4;

        public PointF[] points;

        public MWLocation(float[] _points)
        {

            points = new PointF[4];

            for (int i = 0; i < 4; i++)
            {
                points[i] = new PointF();
                points[i].x = _points[i * 2];
                points[i].y = _points[i * 2 + 1];
            }
            p1 = new PointF();
            p2 = new PointF();
            p3 = new PointF();
            p4 = new PointF();

            p1.x = _points[0];
            p1.y = _points[1];
            p2.x = _points[2];
            p2.y = _points[3];
            p3.x = _points[4];
            p3.y = _points[5];
            p4.x = _points[6];
            p4.y = _points[7];

        }
    }


    class MWResult
    {
        public String text;
        public byte[] bytes;
        public int bytesLength;
        public int type;
        public int subtype;
        public int imageWidth;
        public int imageHeight;
        public Boolean isGS1;
        public MWLocation locationPoints;
        public string encryptedResult;

        public int modulesCountX;
        public int modulesCountY;
        public float moduleSizeX;
        public float moduleSizeY;

        public float skew;
        public bool isKanji;

        public float barcodeWidth;
        public float barcodeHeight;

        public int pdfRows;
        public int pdfColumns;
        public bool pdfIsTruncated;
        public int pdfECLevel;
        public int[] pdfCodewords;

        public MWResult()
        {
            text = null;
            bytes = null;
            bytesLength = 0;
            type = 0;
            subtype = 0;
            isGS1 = false;
            locationPoints = null;
            imageWidth = 0;
            imageHeight = 0;

            modulesCountX = 0;
            modulesCountY = 0;
            moduleSizeX = 0.0F;
            moduleSizeY = 0.0F;

            skew = 0.0F;
            isKanji = false;

            barcodeWidth = 0.0F;
            barcodeHeight = 0.0F;

            pdfRows = 0;
            pdfColumns = 0;
            pdfIsTruncated = false;
            pdfECLevel = 0;
        }

    }

    class MWResults
    {
        public static int MWB_RESULT_FT_BYTES = 0x00000001;
        public static int MWB_RESULT_FT_TEXT = 0x00000002;
        public static int MWB_RESULT_FT_TYPE = 0x00000003;
        public static int MWB_RESULT_FT_SUBTYPE = 0x00000004;
        public static int MWB_RESULT_FT_SUCCESS = 0x00000005;
        public static int MWB_RESULT_FT_ISGS1 = 0x00000006;
        public static int MWB_RESULT_FT_LOCATION = 0x00000007;
        public static int MWB_RESULT_FT_IMAGE_WIDTH = 0x00000008;
        public static int MWB_RESULT_FT_IMAGE_HEIGHT = 0x00000009;
        public static int MWB_RESULT_FT_PARSER_BYTES = 0x0000000A;

        public static int MWB_RESULT_FT_MODULES_COUNT_X = 0x0000000B;
        public static int MWB_RESULT_FT_MODULES_COUNT_Y = 0x0000000C;
        public static int MWB_RESULT_FT_MODULE_SIZE_X = 0x0000000D;
        public static int MWB_RESULT_FT_MODULE_SIZE_Y = 0x0000000E;
        public static int MWB_RESULT_FT_SKEW = 0x0000000F;
        public static int MWB_RESULT_FT_KANJI = 0x00000010;

        public static int MWB_RESULT_FT_BARCODE_WIDTH = 0x00000011;
        public static int MWB_RESULT_FT_BARCODE_HEIGHT = 0x00000012;

        public static int MWB_RESULT_FT_PDF_ROWS = 0x00000020;
        public static int MWB_RESULT_FT_PDF_COLUMNS = 0x00000021;
        public static int MWB_RESULT_FT_PDF_TRUNCATED = 0x00000022;
        public static int MWB_RESULT_FT_PDF_ECLEVEL = 0x00000023;
        public static int MWB_RESULT_FT_PDF_CODEWORDS = 0x00000024;

        public int version;
        public List<MWResult> results;
        public int count;

        public MWResults(byte[] buffer)
        {
            results = new List<MWResult>();
            count = 0;
            version = 0;

            if (buffer[0] != 'M' || buffer[1] != 'W' || buffer[2] != 'R')
            {
                return;
            }

            version = buffer[3];

            count = buffer[4];

            int currentPos = 5;

            for (int i = 0; i < count; i++)
            {

                MWResult result = new MWResult();

                int fieldsCount = buffer[currentPos];
                currentPos++;
                for (int f = 0; f < fieldsCount; f++)
                {
                    int fieldType = buffer[currentPos];
                    int fieldNameLength = buffer[currentPos + 1];
                    int fieldContentLength = 256 * (buffer[currentPos + 3 + fieldNameLength] & 0xFF) + (buffer[currentPos + 2 + fieldNameLength] & 0xFF);
                    String fieldName = null;

                    if (fieldNameLength > 0)
                    {
                        fieldName = Encoding.UTF8.GetString(buffer, currentPos + 2, fieldNameLength);
                    }

                    int contentPos = currentPos + fieldNameLength + 4;
                    float[] locations = new float[8];

                    if (fieldType == MWB_RESULT_FT_TYPE)
                    {
                        result.type = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                        if (fieldType == MWB_RESULT_FT_SUBTYPE)
                    {

                        result.subtype = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                            if (fieldType == MWB_RESULT_FT_ISGS1)
                    {
                        result.isGS1 = BitConverter.ToInt32(buffer, contentPos) == 1;
                    }
                    else
                                if (fieldType == MWB_RESULT_FT_IMAGE_WIDTH)
                    {
                        result.imageWidth = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                                    if (fieldType == MWB_RESULT_FT_IMAGE_HEIGHT)
                    {
                        result.imageHeight = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                                        if (fieldType == MWB_RESULT_FT_LOCATION)
                    {
                        for (int l = 0; l < 8; l++)
                        {
                            locations[l] = BitConverter.ToSingle(buffer, contentPos + l * 4);
                        }
                        result.locationPoints = new MWLocation(locations);
                    }
                    else
                                            if (fieldType == MWB_RESULT_FT_TEXT)
                    {
                        result.text = Encoding.UTF8.GetString(buffer, contentPos, fieldContentLength);
                    }
                    else
                                                if (fieldType == MWB_RESULT_FT_BYTES)
                    {
                        result.bytes = new byte[fieldContentLength];
                        result.bytesLength = fieldContentLength;
                        for (int c = 0; c < fieldContentLength; c++)
                        {
                            result.bytes[c] = buffer[contentPos + c];
                        }
                    }
                    else
                                                    if (fieldType == MWB_RESULT_FT_PARSER_BYTES)
                    {
                        byte[] tmp = new byte[fieldContentLength + 1];

                        for (int c = 0; c < fieldContentLength; c++)
                        {
                            tmp[c] = buffer[contentPos + c];
                        }
                        result.encryptedResult = System.Text.Encoding.UTF8.GetString(tmp);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_MODULES_COUNT_X)
                    {
                        result.modulesCountX = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_MODULES_COUNT_Y)
                    {
                        result.modulesCountY = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_MODULE_SIZE_X)
                    {
                        result.moduleSizeX = BitConverter.ToSingle(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_MODULE_SIZE_Y)
                    {
                        result.moduleSizeY = BitConverter.ToSingle(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_SKEW)
                    {
                        result.skew = BitConverter.ToSingle(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_KANJI)
                    {
                        result.isKanji = (BitConverter.ToInt32(buffer, contentPos) > 0);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_BARCODE_WIDTH)
                    {
                        result.barcodeWidth = BitConverter.ToSingle(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_BARCODE_HEIGHT)
                    {
                        result.barcodeHeight = BitConverter.ToSingle(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_PDF_ROWS)
                    {
                        result.pdfRows = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_PDF_COLUMNS)
                    {
                        result.pdfColumns = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_PDF_TRUNCATED)
                    {
                        result.pdfIsTruncated = (BitConverter.ToInt32(buffer, contentPos) > 0);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_PDF_ECLEVEL)
                    {
                        result.pdfECLevel = BitConverter.ToInt32(buffer, contentPos);
                    }
                    else
                    if (fieldType == MWB_RESULT_FT_PDF_CODEWORDS)
                    {
                        int listLength = BitConverter.ToInt32(buffer, contentPos);
                        result.pdfCodewords = new int[listLength];
                        result.pdfCodewords[0] = listLength;

                        for (int l = 1; l < listLength; l++)
                        {
                            result.pdfCodewords[l] = BitConverter.ToInt32(buffer, contentPos + l * 4);
                        }
                    }

                    currentPos += (fieldNameLength + fieldContentLength + 4);
                }
                results.Add(result);
            }
        }

        public MWResult getResult(int index)
        {
            return results.ElementAt(index);
        }
    }

    class Program
    {

        /** @name Grayscale image size range
         ** @{ */
        public static int MWB_GRAYSCALE_LENX_MIN = 10;
        public static int MWB_GRAYSCALE_LENX_MAX = 3000;
        public static int MWB_GRAYSCALE_LENY_MIN = 10;
        public static int MWB_GRAYSCALE_LENY_MAX = 3000;
        /** @} */

        /**
       * @name Basic return values for API functions
       * @{
       */
        public static int MWB_RT_OK = 0;
        public static int MWB_RT_FAIL = -1;
        public static int MWB_RT_NOT_SUPPORTED = -2;
        public static int MWB_RT_BAD_PARAM = -3;

        /**
         * @name Basic return values for RegisterSDK function
         * @{
         */
        public static int MWB_RTREG_OK = 0;
        public static int MWB_RTREG_INVALID_KEY = -1;
        public static int MWB_RTREG_INVALID_CHECKSUM = -2;
        public static int MWB_RTREG_INVALID_APPLICATION = -3;
        public static int MWB_RTREG_INVALID_SDK_VERSION = -4;
        public static int MWB_RTREG_INVALID_KEY_VERSION = -5;
        public static int MWB_RTREG_INVALID_PLATFORM = -6;
        public static int MWB_RTREG_KEY_EXPIRED = -7;
        /** @} */

        /**
         ** @name    Configuration values for use with MWB_setFlags
         ** @{ */


        /** @brief  Global decoder flags value: apply sharpening on input image
         */
        public static int MWB_CFG_GLOBAL_HORIZONTAL_SHARPENING = 0x1;
        public static int MWB_CFG_GLOBAL_VERTICAL_SHARPENING = 0x2;
        public static int MWB_CFG_GLOBAL_SHARPENING = 0x3;


        /** @brief  Global decoder flags value: apply rotation on input image
         */
        public static int MWB_CFG_GLOBAL_ROTATE90 = 0x04;
        public static int MWB_CFG_GLOBAL_ROTATE180 = 0x08;

        /** @brief  Global decoder flags value: calculate location for 1D barcodeTypes (Code128, Code93, Code39 supported)
          */
        public static int MWB_CFG_GLOBAL_CALCULATE_1D_LOCATION = 0x10;

        /** @brief  Global decoder flags value: fail 1D decode if result is not confirmed by location expanding (Code128, Code93, Code39 supported)
          */
        public static int MWB_CFG_GLOBAL_VERIFY_1D_LOCATION = 0x20;

        /** @brief  Global decoder flags value: fail decode if result is not touching the center of viewfinder (2D + Code128, Code93, Code39 supported)
         * 1D locaiton flags will be enabled automatically with this one
          */
        public static int MWB_CFG_GLOBAL_USE_CENTRIC_SCANNING = 0x40;

        /** @brief  Global decoder flags value: disable some image pre=processing, suitable for devices with weak CPU
          */
        public static int MWB_CFG_GLOBAL_DISABLE_PREPROCESSING = 0x80;


        /** @brief  Code39 decoder flags value: select VIN mode
         */
        //public static int MWB_CFG_CODE39_VIN_MODE = 0x1; //deprecated

        /** @brief  Code39 decoder flags value: require checksum check
         */
        public static int MWB_CFG_CODE39_REQUIRE_CHECKSUM = 0x2;
        /**/

        /** @brief  Code39 decoder flags value: don't require stop symbol - can lead to false results
         */
        public static int MWB_CFG_CODE39_DONT_REQUIRE_STOP = 0x4;
        /**/

        /** @brief  Code39 decoder flags value: decode full ASCII
         */
        public static int MWB_CFG_CODE39_EXTENDED_MODE = 0x8;
        /**/

        /** @brief  Code39 decoder flags value: Try decoding result to CODE32. if failed, Code39 will return
         */
        public static int MWB_CFG_CODE39_CODE32_ENABLED = 0x10;
        /**/

        /** @brief  Code39 decoder flags value: ADD 'A' prefix to Code32 result
         */
        public static int MWB_CFG_CODE39_CODE32_PREFIX = 0x20;
        /**/

        /** @brief  Code93 decoder flags value: decode full ASCII
         */
        public static int MWB_CFG_CODE93_EXTENDED_MODE = 0x8;
        /**/

        /** @brief  UPC/EAN decoder disable addons detection
         */
        public static int MWB_CFG_EANUPC_DISABLE_ADDON = 0x1;
        public static int MWB_CFG_EANUPC_DONT_EXPAND_UPCE = 0x2;
        /**/

        /** @brief  Code25 decoder flags value: require checksum check
         */
        public static int MWB_CFG_CODE25_REQ_CHKSUM = 0x1;
        /**/

        /** @brief  Code11 decoder flags value: require checksum check
         *  MWB_CFG_CODE11_REQ_SINGLE_CHKSUM is set by default
         */
        public static int MWB_CFG_CODE11_REQ_SINGLE_CHKSUM = 0x1;
        public static int MWB_CFG_CODE11_REQ_DOUBLE_CHKSUM = 0x2;
        /**/

        /** @brief  MSI Plessey decoder flags value: require checksum check
         *  MWB_CFG_MSI_REQ_10_CHKSUM is set by default
         */
        public static int MWB_CFG_MSI_REQ_10_CHKSUM = 0x01;
        public static int MWB_CFG_MSI_REQ_1010_CHKSUM = 0x02;
        public static int MWB_CFG_MSI_REQ_11_IBM_CHKSUM = 0x04;
        public static int MWB_CFG_MSI_REQ_11_NCR_CHKSUM = 0x08;
        public static int MWB_CFG_MSI_REQ_1110_IBM_CHKSUM = 0x10;
        public static int MWB_CFG_MSI_REQ_1110_NCR_CHKSUM = 0x20;
        /**/


        /** @brief  Codabar decoder flags value: include start/stop symbols in result
         */
        public static int MWB_CFG_CODABAR_INCLUDE_STARTSTOP = 0x1;
        /**/

        /**
         * @brief   Code128 decoder flag value: select VIN mode
         */
        //public static int MWB_CFG_CODE128_VIN_MODE = 0x1;

        /**
         * @brief   Datamatrix decoder flag value: select VIN mode
         */
        //public static int MWB_CFG_DM_VIN_MODE = 0x1;


        /** @brief  Barcode decoder param types
*/
        public static int MWB_PAR_ID_ECI_MODE = 0x08;
        public static int MWB_PAR_ID_RESULT_PREFIX = 0x10;
        public static int MWB_PAR_ID_VERIFY_LOCATION = 0x20;
        /**/

        /** @brief  Barcode param values
        */

        public static int MWB_PAR_VALUE_ECI_DISABLED = 0x00; //default
        public static int MWB_PAR_VALUE_ECI_ENABLED = 0x01;

        public static int MWB_PAR_VALUE_RESULT_PREFIX_NEVER = 0x00;// default
        public static int MWB_PAR_VALUE_RESULT_PREFIX_ALWAYS = 0x01;
        public static int MWB_PAR_VALUE_RESULT_PREFIX_DEFAULT = 0x02;

        public static int MWB_PAR_VALUE_VERIFY_LOCATION_OFF = 0x00;
        public static int MWB_PAR_VALUE_VERIFY_LOCATION_ON = 0x01;
        /**/

        /** @} */

        /**
         * @name Bit mask identifiers for supported decoder types
         * @{ */
        public static int MWB_CODE_MASK_NONE = 0x00000000;
        public static int MWB_CODE_MASK_QR = 0x00000001;
        public static int MWB_CODE_MASK_DM = 0x00000002;
        public static int MWB_CODE_MASK_RSS = 0x00000004;
        public static int MWB_CODE_MASK_39 = 0x00000008;
        public static int MWB_CODE_MASK_EANUPC = 0x00000010;
        public static int MWB_CODE_MASK_128 = 0x00000020;
        public static int MWB_CODE_MASK_PDF = 0x00000040;
        public static int MWB_CODE_MASK_AZTEC = 0x00000080;
        public static int MWB_CODE_MASK_25 = 0x00000100;
        public static int MWB_CODE_MASK_93 = 0x00000200;
        public static int MWB_CODE_MASK_CODABAR = 0x00000400;
        public static int MWB_CODE_MASK_DOTCODE = 0x00000800;
        public static int MWB_CODE_MASK_11 = 0x00001000;
        public static int MWB_CODE_MASK_MSI = 0x00002000;
        public static int MWB_CODE_MASK_MAXICODE = 0x00004000;
        public static int MWB_CODE_MASK_POSTAL = 0x00008000;
        public static int MWB_CODE_MASK_ALL = 0x0fffffff;
        /** @} */


        /**
         * @name Bit mask identifiers for RSS decoder types
         * @{ */
        public static int MWB_SUBC_MASK_RSS_14 = 0x00000001;
        public static int MWB_SUBC_MASK_RSS_14_STACK = 0x00000002;
        public static int MWB_SUBC_MASK_RSS_LIM = 0x00000004;
        public static int MWB_SUBC_MASK_RSS_EXP = 0x00000008;
        /** @} */

        /**
         * @name Bit mask identifiers for QR decoder types
         * @{ */
        public static int MWB_SUBC_MASK_QR_STANDARD = 0x00000001;
        public static int MWB_SUBC_MASK_QR_MICRO = 0x00000002;
        /** @} */

        /**
         * @name Bit mask identifiers for PDF decoder types
         * @{ */
        public static int MWB_SUBC_MASK_PDF_STANDARD = 0x00000001;
        public static int MWB_SUBC_MASK_PDF_MICRO = 0x00000002;
        /** @} */

        /**
         * @name Bit mask identifiers for 2 of 5 decoder types
         * @{ */
        public static int MWB_SUBC_MASK_C25_INTERLEAVED = 0x00000001;
        public static int MWB_SUBC_MASK_C25_STANDARD = 0x00000002;
        public static int MWB_SUBC_MASK_C25_ITF14 = 0x00000004;
        public static int MWB_SUBC_MASK_C25_IATA = 0x00000008;
        public static int MWB_SUBC_MASK_C25_MATRIX = 0x00000010;
        public static int MWB_SUBC_MASK_C25_COOP = 0x00000020;
        public static int MWB_SUBC_MASK_C25_INVERTED = 0x00000040;
        /** @} */


        /**
        * @name Bit mask identifiers for POSTAL decoder types
        * @{ */
        public static int MWB_SUBC_MASK_POSTAL_POSTNET = 0x00000001;
        public static int MWB_SUBC_MASK_POSTAL_PLANET = 0x00000002;
        public static int MWB_SUBC_MASK_POSTAL_IM = 0x00000004;
        public static int MWB_SUBC_MASK_POSTAL_ROYAL = 0x00000008;
        public static int MWB_SUBC_MASK_POSTAL_AUSTRALIAN = 0x00000010;

        /** @} */

        /**
         * @name Bit mask identifiers for UPC/EAN decoder types
         * @{ */
        public static int MWB_SUBC_MASK_EANUPC_EAN_13 = 0x00000001;
        public static int MWB_SUBC_MASK_EANUPC_EAN_8 = 0x00000002;
        public static int MWB_SUBC_MASK_EANUPC_UPC_A = 0x00000004;
        public static int MWB_SUBC_MASK_EANUPC_UPC_E = 0x00000008;
        public static int MWB_SUBC_MASK_EANUPC_UPC_E1 = 0x00000010;
        /** @} */

        /**
         * @name Bit mask identifiers for 1D scanning direction 
         * @{ */
        public static int MWB_SCANDIRECTION_HORIZONTAL = 0x00000001;
        public static int MWB_SCANDIRECTION_VERTICAL = 0x00000002;
        public static int MWB_SCANDIRECTION_OMNI = 0x00000004;
        public static int MWB_SCANDIRECTION_AUTODETECT = 0x00000008;
        /** @} */

        public static int FOUND_NONE = 0;
        public static int FOUND_DM = 1;
        public static int FOUND_39 = 2;
        public static int FOUND_RSS_14 = 3;
        public static int FOUND_RSS_14_STACK = 4;
        public static int FOUND_RSS_LIM = 5;
        public static int FOUND_RSS_EXP = 6;
        public static int FOUND_EAN_13 = 7;
        public static int FOUND_EAN_8 = 8;
        public static int FOUND_UPC_A = 9;
        public static int FOUND_UPC_E = 10;
        public static int FOUND_128 = 11;
        public static int FOUND_PDF = 12;
        public static int FOUND_QR = 13;
        public static int FOUND_AZTEC = 14;
        public static int FOUND_25_INTERLEAVED = 15;
        public static int FOUND_25_STANDARD = 16;
        public static int FOUND_93 = 17;
        public static int FOUND_CODABAR = 18;
        public static int FOUND_DOTCODE = 19;
        public static int FOUND_128_GS1 = 20;
        public static int FOUND_ITF14 = 21;
        public static int FOUND_11 = 22;
        public static int FOUND_MSI = 23;
        public static int FOUND_25_IATA = 24;
        public static int FOUND_25_MATRIX = 25;
        public static int FOUND_25_COOP = 26;
        public static int FOUND_25_INVERTED = 27;
        public static int FOUND_QR_MICRO = 28;
        public static int FOUND_MAXICODE = 29;
        public static int FOUND_POSTNET = 30;
        public static int FOUND_PLANET = 31;
        public static int FOUND_IMB = 32;
        public static int FOUND_ROYALMAIL = 33;
        public static int FOUND_MICRO_PDF = 34;
        public static int FOUND_32 = 35;
        public static int FOUND_AUSTRALIAN = 36;
        /** @} */

        /**
         * @name Result structure constants
         * @{ */


        /**
         * @name Identifiers for result types
         * @{ */

        public static int MWB_RESULT_TYPE_RAW = 0x00000001;
        public static int MWB_RESULT_TYPE_MW = 0x00000002;
        //public static int MWB_RESULT_TYPE_JSON = 0x00000003; //not yet implemented


        /** @} */


        /**
         * @name Identifiers for result fields types
         * @{ */
        public static int MWB_RESULT_FT_BYTES = 0x00000001;
        public static int MWB_RESULT_FT_TEXT = 0x00000002;
        public static int MWB_RESULT_FT_TYPE = 0x00000003;
        public static int MWB_RESULT_FT_SUBTYPE = 0x00000004;
        public static int MWB_RESULT_FT_SUCCESS = 0x00000005;
        public static int MWB_RESULT_FT_ISGS1 = 0x00000006;
        public static int MWB_RESULT_FT_LOCATION = 0x00000007;
        public static int MWB_RESULT_FT_IMAGE_WIDTH = 0x00000008;
        public static int MWB_RESULT_FT_IMAGE_HEIGHT = 0x00000009;
        public static int MWB_RESULT_FT_PARSER_BYTES = 0x0000000A;

        public static int MWB_RESULT_FT_MODULES_COUNT_X = 0x0000000B;
        public static int MWB_RESULT_FT_MODULES_COUNT_Y = 0x0000000C;
        public static int MWB_RESULT_FT_MODULE_SIZE_X = 0x0000000D;
        public static int MWB_RESULT_FT_MODULE_SIZE_Y = 0x0000000E;
        public static int MWB_RESULT_FT_SKEW = 0x0000000F;
        public static int MWB_RESULT_FT_KANJI = 0x00000010;

        public static int MWB_RESULT_FT_BARCODE_WIDTH = 0x00000011;
        public static int MWB_RESULT_FT_BARCODE_HEIGHT = 0x00000012;

        public static int MWB_RESULT_FT_PDF_ROWS = 0x00000020;
        public static int MWB_RESULT_FT_PDF_COLUMNS = 0x00000021;
        public static int MWB_RESULT_FT_PDF_TRUNCATED = 0x00000022;
        public static int MWB_RESULT_FT_PDF_ECLEVEL = 0x00000023;
        public static int MWB_RESULT_FT_PDF_CODEWORDS = 0x00000024;



        /** @} */

        /**
         * @name Descriptive names of result field types
         * @{ */
        public static string MWB_RESULT_FNAME_BYTES = "Bytes";
        public static string MWB_RESULT_FNAME_TEXT = "Text";
        public static string MWB_RESULT_FNAME_TYPE = "Type";
        public static string MWB_RESULT_FNAME_SUBTYPE = "Subtype";
        public static string MWB_RESULT_FNAME_SUCCESS = "Success";
        public static string MWB_RESULT_FNAME_ISGS1 = "GS1 compliance";
        public static string MWB_RESULT_FNAME_KANJI = "Kanji encoding";
        public static string MWB_RESULT_FNAME_LOCATION = "Location";
        public static string MWB_RESULT_FNAME_IMAGE_WIDTH = "Image Width";
        public static string MWB_RESULT_FNAME_IMAGE_HEIGHT = "Image Height";
        public static string MWB_RESULT_FNAME_PARSER_BYTES = "Parser Input";

        public static string MWB_RESULT_FNAME_MODULES_COUNT_X = "Modules Count X";
        public static string MWB_RESULT_FNAME_MODULES_COUNT_Y = "Modules Count Y";
        public static string MWB_RESULT_FNAME_MODULE_SIZE_X = "Module Size X";
        public static string MWB_RESULT_FNAME_MODULE_SIZE_Y = "Module Size Y";
        public static string MWB_RESULT_FNAME_SKEW = "Skew";

        public static string MWB_RESULT_FNAME_BARCODE_WIDTH = "Barcode Width";
        public static string MWB_RESULT_FNAME_BARCODE_HEIGHT = "Barcode Height";

        public static string MWB_RESULT_FNAME_PDF_ROWS = "PDF Rows";
        public static string MWB_RESULT_FNAME_PDF_COLUMNS = "PDF Columns";
        public static string MWB_RESULT_FNAME_PDF_TRUNCATED = "PDF Truncated";
        public static string MWB_RESULT_FNAME_PDF_ECLEVEL = "PDF ECLevel";
        public static string MWB_RESULT_FNAME_PDF_CODEWORDS = "PDF Codewords";


        /** @} */

        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetLibVersion();
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetSupportedCodes();
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetScanningRect(int codeMask, float left, float top, float width, float height);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetScanningRect(int codeMask, float left, float top, float width, float height);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBregisterSDK_pchar(byte[] key);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetActiveCodes(int codeMask);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetActiveCodes();
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBenableCode(int codeMask);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBdisableCode(int codeMask);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetActiveSubcodes(int codeMask, int subMask);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetActiveSubcodes(int codeMask);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBenableSubcode(int codeMask, int subMask);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBdisableSubcode(int codeMask, int subMask);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBcleanupLib();
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetLastType();
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBscanGrayscaleImage(byte[] gray, int width, int height, byte[] res);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetFlags(int codeMask, int flags);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBenableFlag(int codeMask, int flags);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBdisableFlag(int codeMask, int flags);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetFlags(int codeMask);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetLevel(int level);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetDirection(int direction);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetParam(int codeMask, int paramId, int paramValue);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetParam(int codeMask, int paramId);
        //[DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        //public static extern int MWBvalidateVIN_pchar(byte[] vin, int length); //deprecated
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetBarcodeLocation(float[] points);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetResultType(int resultType);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBgetResultType();
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWBsetDuplicatesTimeout(int timeout);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern void MWBsetDuplicate_pchar(byte[] barcode, int length);

        /**
         * @name Basic return values for API functions
         * @{
         */
        public static int MWP_RT_OK = 0;
        public static int MWP_RT_FAIL = -1;
        public static int MWP_RT_NOT_SUPPORTED = -2;
        public static int MWP_RT_BAD_PARAM = -3;
        public static int MWP_RT_BAD_INPUT = -4;
        /** @} */

        /**
         * @name Bit mask identifiers for supported decoder types
         * @{ */
        public static int MWP_PARSER_MASK_NONE = 0x00000000;
        public static int MWP_PARSER_MASK_GS1 = 0x00000001;
        public static int MWP_PARSER_MASK_IUID = 0x00000002;
        public static int MWP_PARSER_MASK_ISBT = 0x00000004;
        public static int MWP_PARSER_MASK_AAMVA = 0x00000008;
        public static int MWP_PARSER_MASK_HIBC = 0x00000010;
        public static int MWP_PARSER_MASK_SCM = 0x00000020;
        /** @} */

        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWPgetLibVersion();
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern int MWPgetSupportedParsers();
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double MWPgetFormattedText(int parser_type, byte[] p_input, int inputLength, byte[] pp_output);
        [DllImport("BarcodeScanner.dll", CallingConvention = CallingConvention.Cdecl)]
        public static extern double MWPgetJSON(int parser_type, byte[] p_input, int inputLength, byte[] pp_output);

        static public byte[] zeroString(string str)
        {
            string str0 = str + char.MinValue;
            return ASCIIEncoding.ASCII.GetBytes(str0);
        }


        static public int MWBregisterSDK(String key)
        {
            return MWBregisterSDK_pchar(zeroString(key));

        }

        //static public int MWBvalidateVIN(String vin) //deprecated
        //{
        //    return MWBvalidateVIN_pchar(zeroString(vin), vin.Length);

        //}

        static public String MWBscanGrayscaleImageString(byte[] gray, int width, int height)
        {

            byte[] result = new byte[10000];


            int len = MWBscanGrayscaleImage(gray, width, height, result);

            if (len > 0)
            {
                String resStr = "";
                for (int i = 0; i < len; i++)
                    resStr += (char)result[i];
                return resStr;
            }
            else
            {
                return null;
            }

        }

        static public byte[] BitmapDataFromBitmap(Bitmap objBitmap)
        {
            MemoryStream ms = new MemoryStream();
            objBitmap.Save(ms, ImageFormat.Bmp);
            return (ms.GetBuffer());
        }

        static void Main(string[] args)
        {
            int v = MWBgetLibVersion();

            // register your copy of the mobiScan SDK with the given user name / key

            MWBregisterSDK("key");


            // set flags for VIN scan modes
            // MWBsetFlags(MWB_CODE_MASK_39, MWB_CFG_CODE39_VIN_MODE);
            // MWBsetFlags(MWB_CODE_MASK_DM, MWB_CFG_DM_VIN_MODE);

            // Enable verification for code93 barcode type
            // MWBsetParam(MWB_CODE_MASK_93, MWB_PAR_ID_VERIFY_LOCATION, MWB_PAR_VALUE_VERIFY_LOCATION_ON);

            // choose code type or types you want to search for

            // Our sample app is configured by default to search all supported barcodes...
            MWBsetActiveCodes(MWB_CODE_MASK_QR |
                                MWB_CODE_MASK_DM |
                                MWB_CODE_MASK_RSS |
                                MWB_CODE_MASK_39 |
                                MWB_CODE_MASK_EANUPC |
                                MWB_CODE_MASK_128 |
                                MWB_CODE_MASK_PDF |
                                MWB_CODE_MASK_AZTEC |
                                MWB_CODE_MASK_25 |
                                MWB_CODE_MASK_93 |
                                MWB_CODE_MASK_CODABAR |
                                MWB_CODE_MASK_DOTCODE |
                                MWB_CODE_MASK_11 |
                                MWB_CODE_MASK_MSI |
                                MWB_CODE_MASK_MAXICODE |
                                MWB_CODE_MASK_POSTAL
                            );



            // But for better performance, only activate the symbologies your application requires...
            // MWBsetActiveCodes( MWB_CODE_MASK_QR );
            // MWBsetActiveCodes( MWB_CODE_MASK_DM );
            // MWBsetActiveCodes( MWB_CODE_MASK_RSS );
            // MWBsetActiveCodes( MWB_CODE_MASK_39  );
            // MWBsetActiveCodes( MWB_CODE_MASK_EANUPC );
            // MWBsetActiveCodes( MWB_CODE_MASK_128  );
            // MWBsetActiveCodes( MWB_CODE_MASK_PDF );
            // MWBsetActiveCodes( MWB_CODE_MASK_AZTEC );
            // MWBsetActiveCodes( MWB_CODE_MASK_25 );
            // MWBsetActiveCodes( MWB_CODE_MASK_93 );
            // MWBsetActiveCodes( MWB_CODE_MASK_CODABAR );
            // MWBsetActiveCodes( MWB_CODE_MASK_DOTCODE );
            // MWBsetActiveCodes( MWB_CODE_MASK_11 );
            // MWBsetActiveCodes( MWB_CODE_MASK_MSI );
            // MWBsetActiveCodes( MWB_CODE_MASK_MAXICODE );
            // MWBsetActiveCodes( MWB_CODE_MASK_POSTAL );
            // MWBsetActiveCodes( MWB_CODE_MASK_ALL );


            // Our sample app is configured by default to search both directions...
            MWBsetDirection(MWB_SCANDIRECTION_HORIZONTAL | MWB_SCANDIRECTION_VERTICAL);

            /* Parser */
            /*
             * MWPARSER_MASK - Set the desired parser type Available options:
             * MWP_PARSER_MASK_AAMVA
             * MWP_PARSER_MASK_GS1
             * MWP_PARSER_MASK_HIBC
             * MWP_PARSER_MASK_ISBT
             * MWP_PARSER_MASK_IUID
             * MWP_PARSER_MASK_SCM
             * MWP_PARSER_MASK_NONE
             */
            int MWPARSER_MASK = MWP_PARSER_MASK_NONE;
            bool USE_MWPARSER = false;


            Bitmap objBitmap = new Bitmap(args[0]);

            byte[] BitmapData = BitmapDataFromBitmap(objBitmap);

            int width = objBitmap.Width;
            int height = objBitmap.Height;
            int size = width * height;
            byte[] gray = new byte[size];


            for (int x = 0; x < objBitmap.Width; x++)
            {
                for (int y = 0; y < objBitmap.Height; y++)
                {
                    Color pixelColor = objBitmap.GetPixel(x, y);
                    gray[width * y + x] = (byte)(((int)pixelColor.R + (int)pixelColor.G + (int)pixelColor.B) / 3);
                }
            }

            string s = null;



            // set decoder effort level (1 - 5)
            // for live scanning scenarios, a setting between 1 to 3 will suffice
            // levels 4 and 5 are typically reserved for batch scanning 
            MWBsetLevel(2);

            MWBsetResultType(MWB_RESULT_TYPE_MW);
            byte[] p_data = new byte[10000];
            int len = MWBscanGrayscaleImage(gray, width, height, p_data);

            if (len > 0)
            {
                MWResults mwres = new MWResults(p_data);
                MWResult mwResult = null;
                if (mwres != null && mwres.count > 0)
                {
                    mwResult = mwres.getResult(0);

                    /*just decoder w/out parser*/
                    string resultText = "";

                    resultText += "Barcode Type: " + mwResult.type + "\n Decoded Result: " + mwResult.text;

                    Console.WriteLine("Scan result: " + resultText);

                    /*with parser*/
                    if (USE_MWPARSER && MWPARSER_MASK != MWP_PARSER_MASK_NONE)
                    {
                        byte[] pp_data = new byte[10000];

                        string enc = mwResult.encryptedResult;
                        int elen = enc.Length;

                        byte[] enc_p_input = new byte[elen];
                        for (int itmp = 0; itmp < elen; itmp++)
                            enc_p_input[itmp] = (byte)enc[itmp];

                        double fres = MWPgetJSON(MWPARSER_MASK, enc_p_input, elen, pp_data);

                        string resultData = System.Text.Encoding.ASCII.GetString(pp_data).Trim('\0');
                        Console.WriteLine("\n Parsed Result: " + resultData);
                    }
                }
            }
            else
            {
                Console.WriteLine("Scan failed");
            }
        }
    }
}
